#!/bin/bash
# dovecot-maintenance.sh
# 2010-11-23
# bpkroth
#
# This script performs a number of maintenance operations on dovecot mailboxes
# for users.  It is expected to be run daily, but only operates on a single
# letter at a time.  Thus, everyone's mailboxes get operated on every 26 days.
# 
# The tasks it performs are:
# 1) Remove messages that were marked for deletion over N days ago.
# (TODO: We could also consider removing spam or trash files here, but don't currently).
# 2) Rebuild the mailbox indexes to clean out the removed files.
# 3) Rebuild the maildirsize quota calculation to account for the removed files
# and to deal with falling out of sync problems.
# 4) Refresh the full text search index.

DEBUG=true
DRYRUN=false

# The number of days old that a file marked for deletion has to be before we'll remove it.
N=60

# Spread which user accounts we operate on over 26 days.
# lod will loop through [a-z] every 26 days.
doy=`date +%j`
lod=`echo $((($doy % 26) + 97)) | awk '{ printf "%c", $1 }'`

# Set letter of the day to be b for testing.
lod='b'
# And only pull in my accounts.
for theuser in `getent passwd | awk -F: '( $6 ~ "^/mail/'$lod'/" ) { print $1 }' | sort | egrep '^(bpkroth|bkroth|bpktest)$'`; do

#for theuser in `getent passwd | awk -F: '( $6 ~ "^/mail/'$lod'/" ) { print $1 }' | sort`; do

	$DEBUG && echo "Working on '$theuser'"

	# 1) Remove the old files marked for deletion.
	# This is the bit that takes the longest.  So we skip over directories
	# that haven't been changed in a long time (eg: archive mailboxes).
	$DEBUG && echo "Removing $theuser's trashed files older than $N days."
	if $DRYRUN; then
		sudo -u $theuser find /mail/"$lod"/"$theuser"/Maildir \( -type d -name cur -ctime +$N \) -prune -or -type f -name '*:?,*T*' -ctime +$N -ls
	else
		sudo -u $theuser find /mail/"$lod"/"$theuser"/Maildir \( -type d -name cur -ctime +$N \) -prune -or -type f -name '*:?,*T*' -ctime +$N -ls -exec rm "{}" \;
	fi

	# Get the set of the user's mailboxes.
	# NOTE: This currently can't handle mailboxes with " or ' quotes in their name.
	$DEBUG && echo "Getting a list of $theuser's mailboxes."
	mboxes=$(echo 'x LIST * *' | sudo USER="$theuser" -H -u $theuser /usr/sbin/dovecot --exec-mail imap | egrep -e '^\* (LIST|LSUB) \((\\Has(No)?Children)?\) "\/"' -e '^[^*x]' | sed -r -e 's/^\* (LIST|LSUB) \((\\Has(No)?Children)?\) "\/" "(.+)"\s*$/\4/' | grep -v -e '^[*x]' -e '"' -e "'")
	# This will get just their subscribed mailboxes.
	#mboxes=$(echo 'x LSUB * *' | sudo USER="$theuser" -H -u $theuser /usr/sbin/dovecot --exec-mail imap | egrep -e '^\* (LIST|LSUB) \((\\Has(No)?Children)?\) "\/"' -e '^[^*x]' | sed -r -e 's/^\* (LIST|LSUB) \((\\Has(No)?Children)?\) "\/" "(.+)"\s*$/\4/' | grep -v -e '^[*x]' -e '"' -e "'")

	echo -e "$mboxes" | while read mbox; do
		# 2) Update all of the user's dovecot.index files.
		# 4) Update the FTS index.
		$DEBUG && echo "Updating $theuser's indexes for '$mbox'."
		$DEBUG && echo "# x SELECT \""$mbox"\"\nx SEARCH BODY \"blah\"" \| sudo USER="$theuser" -H -u $theuser /usr/sbin/dovecot --exec-mail imap 
		$DRYRUN || echo -e "x SELECT \""$mbox"\"\nx SEARCH BODY \"blah\"" | sudo USER="$theuser" -H -u $theuser /usr/sbin/dovecot --exec-mail imap 
	done

	# 3) Rebuild the maildirsize file.
	# This can be done once for the whole mailbox in our environment.
	$DEBUG && echo "Updating $theuser's maildirsize file."
	sudo -u $theuser test -f "/mail/$lod/$theuser/Maildir/maildirsize" && sudo -u $theuser rm "/mail/$lod/$theuser/Maildir/maildirsize"
	$DEBUG && echo '# x GETQUOTAROOT Inbox' \| sudo USER="$theuser" -H -u $theuser /usr/sbin/dovecot --exec-mail imap 
	$DRYRUN || echo 'x GETQUOTAROOT Inbox' | sudo USER="$theuser" -H -u $theuser /usr/sbin/dovecot --exec-mail imap 

	# Take a breath...
	sleep 1
	echo
done
